# -*- coding: utf-8 -*-
'''
    The venvm is small webtool for manager virtualenv.
    Copyright (C) 2013  Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see {http://www.gnu.org/licenses/}.
'''
import sqlite3 as dba
import os
import re
import codecs


def _read_migration_directory():
    project_dir = os.path.dirname(os.path.abspath(__file__))
    migrate_dir = os.path.join(project_dir, 'migrate')
    regex = re.compile('^.*\.ddl$')

    return sorted([
        (filename.split('.')[0], os.path.join(migrate_dir, filename))
        for filename in os.listdir(migrate_dir)
        if regex.match(filename) is not None
    ])


def _execute_migration(connection, name, filename):
    with codecs.open(filename, 'r', 'utf-8') as fd:
        sql = fd.read()

        try:
            cursor = connection.cursor()
            cursor.execute(sql)
        except dba.Error as e:
            print('Error: %s' % e.message)
            return False
        else:
            cursor.close()
            connection.commit()
            return True


def _check_migration(connection, name):
    try:
        cursor = connection.cursor()
        cursor.execute(
            'select count(*) from history_migration where filename=?',
            (name,)
        )
    except dba.Error as e:
        print('Error: %s' % e.message)
        return False
    else:
        count, = cursor.fetchone()
        cursor.close()
        return (count > 0)


def _register_migration(connection, name):
    try:
        cursor = connection.cursor()
        cursor.execute(
            'insert into history_migration(filename) values(\'%s\')' % name
        )
    except dba.Error as e:
        print('Error: %s' % e.message)
        return False
    else:
        cursor.close()
        connection.commit()
        return True


def _setup_db_check_migrations():
    con = dba.connect('db.data')
    destroy = False

    try:
        migrations = _read_migration_directory()
        cursor = con.cursor()
        cursor.execute('select * from history_migration order by filename')
    except dba.OperationalError as e:
        if e.message == 'no such table: history_migration':
            name, filename = migrations[0]
            if _execute_migration(con, name, filename) is True:
                _register_migration(con, name)
            destroy = False
        else:
            destroy = True
    finally:
        if destroy is False:
            print('execute migrations')
            for serial, filename in migrations:
                print('check migration %s' % serial)
                if _check_migration(con, serial) is False:
                    print('execute migration %s' % serial)
                    if _execute_migration(con, serial, filename) is True:
                        _register_migration(con, serial)
                else:
                    print('migration %s already' % serial)
        else:
            print('destroy migration stage')


def _setup_db_register_models():
    pass


def setup_db():
    print('Check migrations...')
    _setup_db_check_migrations()
    print('Register models...')
